/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_TARGETINFO_H
#define DPC_IMPL_TYPE_TARGETINFO_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_boolean.h"
#include "impl_type_uint32.h"
#include "impl_type_uint8.h"

namespace dpc {
struct TargetInfo {
    ::Float d_tgtlongtdist{ 0.0F };
    ::Float d_tgtlatdist{ 0.0F };
    ::Float v_tgtlongtspd{ 0.0F };
    ::Float v_tgtlatspd{ 0.0F };
    ::Float v_tgtrltspd{ 0.0F };
    ::Boolean is_tgtprsnt{ false };
    ::Float t_tgtdurtm{ 0.0F };
    ::UInt32 indx_tgtid{ 0U };
    ::Float a_tgtlongtaccel{ 0.0F };
    ::Float a_tgtlataccel{ 0.0F };
    ::UInt8 indx_tgttyp{ 0U };
    ::Boolean is_tgtnew{ false };
    ::Boolean is_tgtlnechgflag{ false };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(d_tgtlongtdist);
        fun(d_tgtlatdist);
        fun(v_tgtlongtspd);
        fun(v_tgtlatspd);
        fun(v_tgtrltspd);
        fun(is_tgtprsnt);
        fun(t_tgtdurtm);
        fun(indx_tgtid);
        fun(a_tgtlongtaccel);
        fun(a_tgtlataccel);
        fun(indx_tgttyp);
        fun(is_tgtnew);
        fun(is_tgtlnechgflag);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(d_tgtlongtdist);
        fun(d_tgtlatdist);
        fun(v_tgtlongtspd);
        fun(v_tgtlatspd);
        fun(v_tgtrltspd);
        fun(is_tgtprsnt);
        fun(t_tgtdurtm);
        fun(indx_tgtid);
        fun(a_tgtlongtaccel);
        fun(a_tgtlataccel);
        fun(indx_tgttyp);
        fun(is_tgtnew);
        fun(is_tgtlnechgflag);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("d_tgtlongtdist", d_tgtlongtdist);
        fun("d_tgtlatdist", d_tgtlatdist);
        fun("v_tgtlongtspd", v_tgtlongtspd);
        fun("v_tgtlatspd", v_tgtlatspd);
        fun("v_tgtrltspd", v_tgtrltspd);
        fun("is_tgtprsnt", is_tgtprsnt);
        fun("t_tgtdurtm", t_tgtdurtm);
        fun("indx_tgtid", indx_tgtid);
        fun("a_tgtlongtaccel", a_tgtlongtaccel);
        fun("a_tgtlataccel", a_tgtlataccel);
        fun("indx_tgttyp", indx_tgttyp);
        fun("is_tgtnew", is_tgtnew);
        fun("is_tgtlnechgflag", is_tgtlnechgflag);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("d_tgtlongtdist", d_tgtlongtdist);
        fun("d_tgtlatdist", d_tgtlatdist);
        fun("v_tgtlongtspd", v_tgtlongtspd);
        fun("v_tgtlatspd", v_tgtlatspd);
        fun("v_tgtrltspd", v_tgtrltspd);
        fun("is_tgtprsnt", is_tgtprsnt);
        fun("t_tgtdurtm", t_tgtdurtm);
        fun("indx_tgtid", indx_tgtid);
        fun("a_tgtlongtaccel", a_tgtlongtaccel);
        fun("a_tgtlataccel", a_tgtlataccel);
        fun("indx_tgttyp", indx_tgttyp);
        fun("is_tgtnew", is_tgtnew);
        fun("is_tgtlnechgflag", is_tgtlnechgflag);
    }

    friend bool operator==(const ::dpc::TargetInfo& lhs, const ::dpc::TargetInfo& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.d_tgtlongtdist - rhs.d_tgtlongtdist)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.d_tgtlatdist - rhs.d_tgtlatdist)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.v_tgtlongtspd - rhs.v_tgtlongtspd)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.v_tgtlatspd - rhs.v_tgtlatspd)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.v_tgtrltspd - rhs.v_tgtrltspd)) < DBL_EPSILON) && (lhs.is_tgtprsnt == rhs.is_tgtprsnt) && (fabs(static_cast<double>(lhs.t_tgtdurtm - rhs.t_tgtdurtm)) < DBL_EPSILON) && (lhs.indx_tgtid == rhs.indx_tgtid) && (fabs(static_cast<double>(lhs.a_tgtlongtaccel - rhs.a_tgtlongtaccel)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.a_tgtlataccel - rhs.a_tgtlataccel)) < DBL_EPSILON) && (lhs.indx_tgttyp == rhs.indx_tgttyp) && (lhs.is_tgtnew == rhs.is_tgtnew) && (lhs.is_tgtlnechgflag == rhs.is_tgtlnechgflag);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_TARGETINFO_H
